#ifndef TensorImpl_ref_cpp_H_
#define TensorImpl_ref_cpp_H_

#include "backend/TensorImpl.hpp"
#include "data/Tensor.hpp"
#include "utils/Registrar.hpp"
#include "utils/Types.h"

namespace Aidge {
template <class T>
class TensorImpl_ref_cpp : public TensorImpl {
   private:
    const Tensor &mTensor;  // Impl needs to access Tensor information, but is not
                            // supposed to change it!
    std::vector<T> mData;

   public:
    static constexpr const char *Backend = "cpu";

    TensorImpl_ref_cpp(const Tensor &tensor) : TensorImpl(Backend), mTensor(tensor) {}

    bool operator==(const TensorImpl &otherImpl) const override final {
        std::size_t i = 0;
        for (; i < mTensor.size() &&
               mData[i] == reinterpret_cast<const TensorImpl_ref_cpp<T> &>(otherImpl).data()[i];
             ++i) {
        }
        return i == mTensor.size();
    }

    static std::unique_ptr<TensorImpl_ref_cpp> create(const Tensor &tensor) {
        return std::make_unique<TensorImpl_ref_cpp<T>>(tensor);
    }

    // native interface
    const std::vector<T> &data() const { return mData; }

    std::size_t scalarSize() const override { return sizeof(T); }

    void copy(const void *src, NbElts_t length) override {
        std::copy(static_cast<const T *>(src), static_cast<const T *>(src) + length,
                  static_cast<T *>(rawPtr()));
    }

    void *rawPtr() override {
        lazyInit(mData);
        return mData.data();
    };

    virtual ~TensorImpl_ref_cpp() = default;

    void setRawPtr(void *ptr) override final {
        T *newPtr = static_cast<T *>(ptr);
        mData = std::vector<T>(newPtr, newPtr + (mTensor.size() * sizeof(T)));
    };

   private:
    void lazyInit(std::vector<T> &data) {
        assert(mTensor.dataType() == NativeType<T>::type);

        if (data.size() != mTensor.size()) data.resize(mTensor.size());
    }
};

namespace {
static Registrar<Tensor> registrarTensorImpl_ref_cpp_Float32(
        {"cpu", DataType::Float32}, Aidge::TensorImpl_ref_cpp<float>::create);
static Registrar<Tensor> registrarTensorImpl_ref_cpp_Int32({"cpu", DataType::Int32},
                                                           Aidge::TensorImpl_ref_cpp<int>::create);
}  // namespace
}  // namespace Aidge

#endif /* TensorImpl_ref_cpp_H_ */
