#ifndef AddImpl_ref_cpp_forward_kernel_H_
#define AddImpl_ref_cpp_forward_kernel_H_

#include "utils/Registrar.hpp"

#include "operator/AddImpl.hpp"

namespace Aidge {

template <class I1, class O>
void AddImpl1I_ref_cpp_forward_kernel(const std::size_t inputLength, const void* input1_, void* output_) {
    // FIXME: missing Add parameters as arguments
    const I1* input1 = static_cast<const I1*>(input1_);
    O* output = static_cast<O*>(output_);

    for (std::size_t oIndex = 0; oIndex < inputLength; ++oIndex) {
        output[oIndex] = input1[oIndex];
    }
}

template <class I1, class I2, class O>
void AddImpl2I_ref_cpp_forward_kernel(const std::size_t inputLength, const void* input1_, const void* input2_,
                                      void* output_) {
    // FIXME: missing Add parameters as arguments
    const I1* input1 = static_cast<const I1*>(input1_);
    const I2* input2 = static_cast<const I2*>(input2_);
    O* output = static_cast<O*>(output_);

    for (std::size_t oIndex = 0; oIndex < inputLength; ++oIndex) {
        output[oIndex] = input1[oIndex] + input2[oIndex];
    }
}

template <class I1, class I2, class I3, class O>
void AddImpl3I_ref_cpp_forward_kernel(const std::size_t inputLength, const void* input1_, const void* input2_,
                                      const void* input3_, void* output_) {
    // FIXME: missing Add parameters as arguments
    const I1* input1 = static_cast<const I1*>(input1_);
    const I2* input2 = static_cast<const I2*>(input2_);
    const I3* input3 = static_cast<const I3*>(input3_);
    O* output = static_cast<O*>(output_);

    for (std::size_t oIndex = 0; oIndex < inputLength; ++oIndex) {
        output[oIndex] = input1[oIndex] + input2[oIndex] + input3[oIndex];
    }
}

namespace {
static Registrar<AddImplForward_ref_cpp<1>> registrarAddImpl1IForward_ref_cpp_Float32(
        {DataType::Float32, DataType::Float32}, Aidge::AddImpl1I_ref_cpp_forward_kernel<float, float>);
static Registrar<AddImplForward_ref_cpp<1>> registrarAddImpl1IForward_ref_cpp_Int32(
        {DataType::Int32, DataType::Int32}, Aidge::AddImpl1I_ref_cpp_forward_kernel<int, int>);
static Registrar<AddImplForward_ref_cpp<1>> registrarAddImpl1IForward_ref_cpp_Float64(
        {DataType::Float64, DataType::Float64}, Aidge::AddImpl1I_ref_cpp_forward_kernel<double, double>);

static Registrar<AddImplForward_ref_cpp<2>> registrarAddImpl2IForward_ref_cpp_Float32(
        {DataType::Float32, DataType::Float32, DataType::Float32},
        Aidge::AddImpl2I_ref_cpp_forward_kernel<float, float, float>);
static Registrar<AddImplForward_ref_cpp<2>> registrarAddImpl2IForward_ref_cpp_Int32(
        {DataType::Int32, DataType::Int32, DataType::Int32}, Aidge::AddImpl2I_ref_cpp_forward_kernel<int, int, int>);
static Registrar<AddImplForward_ref_cpp<2>> registrarAddImpl2IForward_ref_cpp_Float64(
        {DataType::Float64, DataType::Float64, DataType::Float64}, Aidge::AddImpl2I_ref_cpp_forward_kernel<double, double, double>);

static Registrar<AddImplForward_ref_cpp<3>> registrarAddImpl3IForward_ref_cpp_Float32(
        {DataType::Float32, DataType::Float32, DataType::Float32, DataType::Float32},
        Aidge::AddImpl3I_ref_cpp_forward_kernel<float, float, float, float>);
static Registrar<AddImplForward_ref_cpp<3>> registrarAddImpl3IForward_ref_cpp_Int32(
        {DataType::Int32, DataType::Int32, DataType::Int32, DataType::Int32},
        Aidge::AddImpl3I_ref_cpp_forward_kernel<int, int, int, int>);
static Registrar<AddImplForward_ref_cpp<3>> registrarAddImpl3IForward_ref_cpp_Float64(
        {DataType::Float64, DataType::Float64, DataType::Float64, DataType::Float64},
        Aidge::AddImpl3I_ref_cpp_forward_kernel<double, double, double, double>);
}  // namespace
}  // namespace Aidge

#endif /* AddImpl_ref_cpp_forward_kernel_H_ */
