#include <catch2/catch_test_macros.hpp>

#include "operator/GenericOperator.hpp"
#include "graph/GraphView.hpp"
#include <cstddef>

using namespace Aidge;

TEST_CASE("[core/operators] GenericOp(add & get parameters)", "[Operator]") {
    SECTION("INT") {
        GenericOperator_Op Testop("TestOp", 1, 1, 1);
        int value = 5;
        const char* key = "intParam";
        Testop.addParameter<int>(key, value);
        REQUIRE(Testop.getParameter<int>(key) == value);
    }
    SECTION("LONG") {
        GenericOperator_Op Testop("TestOp", 1, 1, 1);
        long value = 3;
        const char* key = "longParam";
        Testop.addParameter<long>(key, value);
        REQUIRE(Testop.getParameter<long>(key) == value);
    }
    SECTION("FLOAT") {
        GenericOperator_Op Testop("TestOp", 1, 1, 1);
        float value = 2.0;
        const char* key = "floatParam";
        Testop.addParameter<float>(key, value);
        REQUIRE(Testop.getParameter<float>(key) == value);
    }
    SECTION("VECTOR<INT>") {
        GenericOperator_Op Testop("TestOp", 1, 1, 1);
        std::vector<int> value = {1, 2};
        const char* key = "vect";
        Testop.addParameter<std::vector<int>>(key, value);

        REQUIRE(Testop.getParameter<std::vector<int>>(key).size() == value.size());
        for (std::size_t i=0; i < value.size(); ++i){
            REQUIRE(Testop.getParameter<std::vector<int>>(key)[i] == value[i]);
        } 
    }
    SECTION("MULTIPLE PARAMS") {
        /*
        Goal : Test that the offsets are well done by adding different parameters with different size.
        */
        GenericOperator_Op Testop("TestOp", 1, 1, 1);
        Testop.addParameter<long>("longParam", 3);
        Testop.addParameter<float>("floatParam", 2.0);
        Testop.addParameter<uint8_t>("uint8Param", 5);
        Testop.addParameter<long long>("llParam", 10);
        REQUIRE(Testop.getParameter<long>("longParam") == 3);
        REQUIRE(Testop.getParameter<float>("floatParam") == 2.0);
        REQUIRE(Testop.getParameter<uint8_t>("uint8Param") == 5);
        REQUIRE(Testop.getParameter<long long>("llParam") == 10);
    }
}

TEST_CASE("[core/operator] GenericOp(type check)", "[.ass]") {
    SECTION("WRONG TYPE FOR GETTER") {
        GenericOperator_Op Testop("TestOp", 1, 1, 1);
        Testop.addParameter<long>("longParam", 3);

        // This line should raise a failled assert
        REQUIRE_THROWS(Testop.getParameter<int>("longParameter"));
    }
}
