#include <iostream>
#include <map>
#include <memory>
#include <vector>
#include <utility>
#include <cassert>

#include <catch2/catch_test_macros.hpp>
//test
#include "graphmatching/SeqStm.hpp"
#include "graphmatching/NodeRegex.hpp"
//use
#include "backend/OperatorImpl.hpp"
#include "operator/GenericOperator.hpp"
#include "operator/Producer.hpp"

using namespace Aidge;

TEST_CASE("Create good init SeqStm", "[SeqStm]") {
    //init all iniput for SeqStm


    int stmIdx = 0;
    //matrix that in B->C
    std::vector<std::vector<int>> transitionMatrix { 
        { -1, 1, -1 },
        { -1, -1, 2 },
        { -1, -1, -1 } };

    //std::cout << transitionMatrix.size() << "\n";
    // init the nodes Regex map 
    std::map<std::string,NodeRegex*> nodesRegex ;
    std::vector<std::string> nodeTypeKey {"A","B","C"};
    for (const std::string& key : nodeTypeKey) {
        nodesRegex[key] = new NodeRegex(key);
    }
    //

    std::map<NodeTypeKey,int> typeToIdxTransition;
    std::vector<NodeTypeKey> nodeTypeCommonTag {{"A",""},{"B",""},{"C",""}};
    //init nodeTypeCommonTag
    int idx = 0;
    for (const NodeTypeKey& key : nodeTypeCommonTag) {
        typeToIdxTransition[key] = idx;
        idx += 1;
    }

    int actSt = 0;
    std::set<NodeTmp> allNodeValidated;
    std::set<NodeTmp> allNodeTested;
    std::set<std::pair<NodeTmp,std::string>> allCommonNode;
    bool stmIsValid =false;


    SeqStm stm( 
        stmIdx,
        transitionMatrix,
        nodesRegex,
        typeToIdxTransition,
        actSt, 
        allNodeValidated,
        allNodeTested,
        allCommonNode,
        stmIsValid);

    REQUIRE(stm.getStmIdx() == 0);
    REQUIRE(stm.isValid() == false);
    REQUIRE(stm.getAllCommonNode().size()     == 0);
    REQUIRE(stm.getAllNodeTested().size()     == 0);
    REQUIRE(stm.getAllNodeValidated().size()  == 0);
}

TEST_CASE("Test testNode function", "[SeqStm]") {

    int stmIdx = 0;
    std::map<NodeTypeKey,int> typeToIdxTransition;
    std::vector<NodeTypeKey> nodeTypeCommonTag {{"A",""},{"B",""},{"C",""}};
    //init nodeTypeCommonTag
    int idx = 0;
    for (const NodeTypeKey& key : nodeTypeCommonTag) {
        typeToIdxTransition[key] = idx;
        idx += 1;
    }
    //matrix that in B->C
    std::vector<std::vector<int>> transitionMatrix { 
        { -1, 1, -1 },
        { -1, -1, 2 },
        { -1, -1, -1 } };

    //std::cout << transitionMatrix.size() << "\n";
    // init the nodes Regex map 
    std::map<std::string,NodeRegex*> nodesRegex ;
    std::vector<std::string> nodeTypeKey {"A","B","C"};
    for (const std::string& key : nodeTypeKey) {
        nodesRegex[key] = new NodeRegex(key);
    }
    //
    int actSt = 0;
    std::set<NodeTmp> allNodeValidated;
    std::set<NodeTmp> allNodeTested;
    std::set<std::pair<NodeTmp,std::string>> allCommonNode;
    bool stmIsValid =false;

    SeqStm stm( 
        stmIdx,
        transitionMatrix,
        nodesRegex,
        typeToIdxTransition,
        actSt, 
        allNodeValidated,
        allNodeTested,
        allCommonNode,
        stmIsValid);
    REQUIRE(stm.getStmIdx() == 0);
    //test a node 
    std::shared_ptr<Node> nodeB = GenericOperator("B", 1, 1, 1);
    std::shared_ptr<Node> nodeC = GenericOperator("C", 1, 1, 1);

    //set use to test the state of the smt
    std::set<NodeTmp> testAllNodeTested;
    std::set<NodeTmp> testAllNodeValidated;

    stm.testNode(nodeB);
    REQUIRE(stm.isValid() == false);
    REQUIRE(stm.getState() == 1);
    REQUIRE(stm.isStmBlocked() == false);
    testAllNodeTested.insert(nodeB);
    testAllNodeValidated.insert(nodeB);
    REQUIRE(stm.getAllNodeTested() == testAllNodeTested);
    REQUIRE(stm.getAllNodeValidated() == testAllNodeValidated);


    stm.testNode(nodeC);
    REQUIRE(stm.isValid() == true);
    REQUIRE(stm.getState() == 2);
    REQUIRE(stm.isStmBlocked() == false);
    testAllNodeTested.insert(nodeC);
    testAllNodeValidated.insert(nodeC);
    REQUIRE(stm.getAllNodeTested() == testAllNodeTested);
    REQUIRE(stm.getAllNodeValidated() == testAllNodeValidated);

    stm.testNode(nodeC);
    REQUIRE(stm.isValid() == true);
    REQUIRE(stm.getState() == -1);
    REQUIRE(stm.isStmBlocked() == true);
    REQUIRE(stm.getAllNodeTested() == testAllNodeTested);
    REQUIRE(stm.getAllNodeValidated() == testAllNodeValidated);
}